package com.hongyegroup.cpt_rewards.ui

import android.content.Intent
import android.os.Bundle
import android.text.Editable
import android.text.TextUtils
import android.text.TextWatcher
import android.util.TypedValue
import android.view.View
import android.view.ViewGroup
import androidx.databinding.ViewDataBinding
import com.guadou.cs_cptservices.YYConstants
import com.guadou.lib_baselib.base.activity.BaseVDBActivity
import com.guadou.lib_baselib.bean.DataBindingConfig
import com.guadou.lib_baselib.cache.ACache
import com.guadou.lib_baselib.ext.vertical
import com.guadou.lib_baselib.font_text_view.TextViewLight
import com.guadou.lib_baselib.utils.CommUtils
import com.guadou.lib_baselib.utils.KeyboardUtils
import com.guadou.ninegrid.flownine.FlowLayout
import com.hongyegroup.cpt_rewards.BR
import com.hongyegroup.cpt_rewards.R
import com.hongyegroup.cpt_rewards.databinding.ActivityRewardsSearchBinding
import com.hongyegroup.cpt_rewards.mvvm.RewardsSearchViewModel
import com.luck.picture.lib.tools.ToastUtils
import java.util.*
import kotlin.collections.ArrayList

class RewardsSearchActivity : BaseVDBActivity<RewardsSearchViewModel, ActivityRewardsSearchBinding>() {

    val mClickProxy by lazy { ClickProxy() }

    companion object {

        fun startInstance() {
            val context = CommUtils.getContext()
            val intent = Intent(context, RewardsSearchActivity::class.java)
            intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK)
            context.startActivity(intent)
        }

    }

    override fun init(savedInstanceState: Bundle?) {

        initView()

        initListener()

    }

    private fun initListener() {

        mBinding.etSearchContent.addTextChangedListener(object :TextWatcher{
            override fun beforeTextChanged(p0: CharSequence?, p1: Int, p2: Int, p3: Int) {

            }

            override fun onTextChanged(charSequence: CharSequence?, p1: Int, p2: Int, p3: Int) {
                if (charSequence?.length == 0) {
                    mBinding.ivSearchDelete.setVisibility(View.GONE)
                    mViewModel.mSearchStr = ""
                } else {
                    mBinding.ivSearchDelete.setVisibility(View.VISIBLE)
                    mViewModel.mSearchStr = charSequence.toString().trim { it <= ' ' }
                }
            }

            override fun afterTextChanged(p0: Editable?) {

            }
        })

    }

    private fun initView() {

        for (index in 0..8){

            mViewModel.mRewardsData.add("")

        }

        mBinding.recyclerViewRewardsSearch.vertical().adapter=mViewModel.mRewardsAdapter
        mViewModel.mRewardsAdapter.notifyDataSetChanged()

        //初始化搜索历史数据
        addSearchHistory()

    }

    override fun getDataBindingConfig(): DataBindingConfig {

        return DataBindingConfig(R.layout.activity_rewards_search, BR.viewModel, mViewModel)
            .addBindingParams(BR.click,mClickProxy)


    }

    //添加搜索数据到缓存中
    private fun addSearchStrToACache() {

        //如果缓存中有搜索的内容，先移除，再添加
        var searchList = ACache.get(CommUtils.getContext()).getAsObject(YYConstants.CACHE_PROMOTION_REWARDS_LIST_SEARCH_HISTORY)

        //移除相同搜索内容
        if (searchList != null && (searchList as ArrayList<String>).size > 0) {
            for (i in searchList.indices) {
                if (searchList[i] == mViewModel.mSearchStr) {
                    searchList.removeAt(i)
                    break
                }
            }
        } else {
            searchList = ArrayList<String>()
        }

        //添加搜索内容
        if (searchList.size >= 5) {
            searchList.removeAt(0)
        }
        searchList.add(mViewModel.mSearchStr)
        ACache.get(CommUtils.getContext()).put(YYConstants.CACHE_PROMOTION_REWARDS_LIST_SEARCH_HISTORY, searchList)
    }

    //添加历史记录View
    private fun addSearchHistory() {
        val searchList = ACache.get(CommUtils.getContext()).getAsObject(YYConstants.CACHE_PROMOTION_REWARDS_LIST_SEARCH_HISTORY)
        if (searchList != null && (searchList as ArrayList<String>).size > 0) {
            mBinding.tvRewardsSearchRecentSearch.setVisibility(View.VISIBLE)
            mBinding.flowLayoutRewardsSearchRecentSearch.setVisibility(View.VISIBLE)

            //反转List
            Collections.reverse(searchList)

            //先移除所有控件
            mBinding.flowLayoutRewardsSearchRecentSearch.removeAllViews()

            //再添加View
            for (i in searchList.indices) {
                val searchStr = searchList[i]
                val textView = TextViewLight(mContext)
                textView.setTextColor(CommUtils.getColor(R.color.black))
                textView.setTextSize(TypedValue.COMPLEX_UNIT_SP, 14F)
                textView.setText(searchStr)
                textView.setBackground(CommUtils.getDrawable(R.drawable.shape_page_gray_bg_border_line_3conner))
                val layoutParams = FlowLayout.LayoutParams(ViewGroup.LayoutParams.WRAP_CONTENT, ViewGroup.LayoutParams.WRAP_CONTENT)
                textView.setPadding(CommUtils.dip2px(20), CommUtils.dip2px(4), CommUtils.dip2px(20), CommUtils.dip2px(4))
                layoutParams.setMargins(0, 0, CommUtils.dip2px(7), CommUtils.dip2px(5))
                textView.setLayoutParams(layoutParams)
                textView.setOnClickListener(View.OnClickListener { //跳转搜索结果界面
                    toSearchResult(searchStr)
                })
                //添加到流式布局中
                mBinding.flowLayoutRewardsSearchRecentSearch.addView(textView)
            }
        } else {
            mBinding.tvRewardsSearchRecentSearch.setVisibility(View.GONE)
            mBinding.flowLayoutRewardsSearchRecentSearch.setVisibility(View.GONE)
        }
    }

    //跳转到搜索结果界面
    private fun toSearchResult(searchStr: String?) {
        mBinding.etSearchContent.setText(searchStr)
        search()
    }

    //搜索
    private fun search() {
        if (TextUtils.isEmpty(mViewModel.mSearchStr)) {

            ToastUtils.s(CommUtils.getContext(),"Please input search content")
            return
        }

        KeyboardUtils.hideSoftInput(mBinding.etSearchContent)
//        startWithPop(RewardsSearchResultFragment.newInstance(mSearchStr))

        //添加搜索历史到缓存
        addSearchStrToACache()
        //添加到历史记录View
        addSearchHistory()
    }

    override fun startObserve() {

    }

    /**
     * DataBinding事件处理
     */
    inner class ClickProxy {

        //处理搜索逻辑
        fun handleSearch(){

            toSearchResult(mViewModel.mSearchStr)

        }

        //清除搜索
        fun clearSearch(){

            mBinding.etSearchContent.setText("")

        }

        //返回按钮
        fun finishActivity(){

            mActivity.finish()

        }

    }

}